-- File ECOA-File.ads

-- Standard ECOA Types
with ECOA;

package ECOA.File is

-- Record type
type Reference is
   record
      descriptor : ECOA.Signed_32_Type;
   end record;

-- Fixed Array type
type FileName_Index is range 0..256-1;
type FileName is array
    (FileName_Index) of ECOA.Character_8_Type;

-- Enum type
type FileFlag is new ECOA.Unsigned_16_Type;
FileFlag_READ : constant FileFlag := 0;
FileFlag_WRITE : constant FileFlag := 1;
FileFlag_READWRITE : constant FileFlag := 2;
FileFlag_APPEND : constant FileFlag := 8;
FileFlag_NONBLOCK : constant FileFlag := 128;
FileFlag_CREATE : constant FileFlag := 256;
FileFlag_TRUNCATE : constant FileFlag := 512;
FileFlag_EXCLUSIVE : constant FileFlag := 1024;
FileFlag_NOCTTY : constant FileFlag := 2048;
FileFlag_PRIVATE : constant FileFlag := 4096;
FileFlag_LARGEFILE : constant FileFlag := 8192;
FileFlag_BINARY : constant FileFlag := 16384;

-- Enum type
type SeekBase is new ECOA.Unsigned_8_Type;
SeekBase_SEEK_SET : constant SeekBase := 0;
SeekBase_SEEK_CUR : constant SeekBase := 1;
SeekBase_SEEK_END : constant SeekBase := 2;

-- Enum type
type Permission is new ECOA.Unsigned_8_Type;
Permission_READ : constant Permission := 4;
Permission_WRITE : constant Permission := 2;
Permission_EXECUTE : constant Permission := 1;
Permission_READEXEC : constant Permission := 5;
Permission_ALL : constant Permission := 7;

-- Record type
type Permissions is
   record
      Owner : Permission;
      Group : Permission;
      Others : Permission;
   end record;

-- Record type
type FileSpec is
   record
      FileName : FileName;
      Access : FileFlag;
      CreatePermissions : Permissions;
   end record;

end ECOA.File;

