/* File ECOA__File.hpp */

#if !defined(_ECOA__FILE_HH)
#define _ECOA__FILE_HH

#include "ECOA.hpp" /* Fundamental types declared for the platform */

namespace ECOA {

namespace File {

/* Record type */
typedef struct
{
   ECOA::int32 descriptor;
} Reference;

/* Fixed array type */
const ECOA::uint32 FileName_MAXSIZE  = 256;
typedef ECOA::char8 FileName[FileName_MAXSIZE];

/* Enum type */
struct FileFlag
{
   ECOA::uint16 value;
   enum EnumValues {
      READ = 0,
      WRITE = 1,
      READWRITE = 2,
      APPEND = 8,
      NONBLOCK = 128,
      CREATE = 256,
      TRUNCATE = 512,
      EXCLUSIVE = 1024,
      NOCTTY = 2048,
      PRIVATE = 4096,
      LARGEFILE = 8192,
      BINARY = 16384
   };
inline void operator = (ECOA::uint16 i) { value = i; }
inline operator ECOA::uint16() const { return value; }
};

/* Enum type */
struct SeekBase
{
   ECOA::uint8 value;
   enum EnumValues {
      SEEK_SET = 0,
      SEEK_CUR = 1,
      SEEK_END = 2
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Enum type */
struct Permission
{
   ECOA::uint8 value;
   enum EnumValues {
      READ = 4,
      WRITE = 2,
      EXECUTE = 1,
      READEXEC = 5,
      ALL = 7
   };
inline void operator = (ECOA::uint8 i) { value = i; }
inline operator ECOA::uint8() const { return value; }
};

/* Record type */
typedef struct
{
   Permission Owner;
   Permission Group;
   Permission Others;
} Permissions;

/* Record type */
typedef struct
{
   FileName FileName;
   FileFlag Access;
   Permissions CreatePermissions;
} FileSpec;

} /* File */

} /* ECOA */

#endif  /* _ECOA__FILE_HH */
